library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
-------------------------------------------------------------------
--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
library UNISIM;  
use UNISIM.VComponents.all;
--------------------------------------------------------------------
entity Clock_generator is
    Port ( clock_in 		: in std_logic;	-- input clock = 80 MHz
           clock40		: out std_logic;
			  clock40_TDC	: out std_logic;			  
			  clock80		: out std_logic;	-- This clock is programmable from CLKFX		  		  		  
			  DCM_1_Locked	: out std_logic;  -- high = locked
           RESET 			: in std_logic);
end Clock_generator;
																	 
architecture Behavioral of Clock_generator is
--clock management
-- Components Declarations:
component BUFG is
	port (
			I : in std_logic;
			O : out std_logic
			);
end component;

component IBUFG is
	port (
			I : in std_logic;
			O : out std_logic
			);
end component;

component DCM is
	port ( CLKIN 	: in std_logic;
			CLKFB 	: in std_logic;
			DSSEN 	: in std_logic;
			PSINCDEC : in std_logic;
			PSEN 		: in std_logic;
			PSCLK 	: in std_logic;
			RST 		: in std_logic;
			CLK0 		: out std_logic;
			CLK90 	: out std_logic;
			CLK180 	: out std_logic;
			CLK270 	: out std_logic;
			CLK2X 	: out std_logic;
			CLK2X180 : out std_logic;
			CLKDV 	: out std_logic;
			CLKFX 	: out std_logic;
			CLKFX180 : out std_logic;
			LOCKED 	: out std_logic;
			PSDONE 	: out std_logic;
			STATUS 	: out std_logic_vector(7 downto 0)
			);
end component;

-- Attributes for clock buffers
attribute DLL_FREQUENCY_MODE 	: string;
attribute DFS_FREQUENCY_MODE 	: string;
attribute STARTUP_WAIT 			: string;
attribute DUTY_CYCLE_CORRECTION : string;
attribute CLKDV_DIVIDE 			: string;
attribute CLKOUT_PHASE_SHIFT 	: string;
attribute PHASE_SHIFT 			: integer;
attribute CLKFX_MULTIPLY 		: integer;
attribute CLKFX_DIVIDE 			: integer;
attribute CLKIN_PERIOD 			: string;
--attribute setting
--------- Clock40 setting
attribute DLL_FREQUENCY_MODE 		of DCM_1: label is "LOW";
attribute DFS_FREQUENCY_MODE 		of DCM_1: label is "LOW";
attribute STARTUP_WAIT 				of DCM_1: label is "TRUE";
attribute DUTY_CYCLE_CORRECTION 	of DCM_1: label is "TRUE";
attribute CLKDV_DIVIDE 				of DCM_1: label is "2.0";		--  Can be any interger are 1.5, 2, 2.5, 3, 3.5, 4, 4.5, 5, 5.5, 6, 6.5,
 																					--  7, 7.5, 8, 9, 10, 11, 12, 13, 14, 15 and 16.
attribute CLKOUT_PHASE_SHIFT 		of DCM_1: label is "FIXED";
attribute PHASE_SHIFT 				of DCM_1: label is 0; 			--	(Any value from -255 to 255)
--attribute CLKFX_MULTIPLY 			of DCM_1: label is 1; 			--  Can be any integer from 1 to 32
attribute CLKFX_MULTIPLY 			of DCM_1: label is 5; 		--  Can be any integer from 2 to 32
attribute CLKFX_DIVIDE 				of DCM_1: label is 2;   		--  Can be any integer from 1 to 32
attribute CLKIN_PERIOD 				of DCM_1: label is "12.5";		--  Specify period of input clock

--------- Clock40_TDC setting
attribute DLL_FREQUENCY_MODE 		of DCM_2: label is "LOW";
attribute DFS_FREQUENCY_MODE 		of DCM_2: label is "LOW";
attribute STARTUP_WAIT 				of DCM_2: label is "TRUE";
attribute DUTY_CYCLE_CORRECTION 	of DCM_2: label is "TRUE";
attribute CLKDV_DIVIDE 				of DCM_2: label is "2.0";		--  Can be any interger are 1.5, 2, 2.5, 3, 3.5, 4, 4.5, 5, 5.5, 6, 6.5,
 																					--  7, 7.5, 8, 9, 10, 11, 12, 13, 14, 15 and 16.
attribute CLKOUT_PHASE_SHIFT 		of DCM_2: label is "FIXED";
attribute PHASE_SHIFT 				of DCM_2: label is 0; 			--	(Any value from -255 to 255)
--attribute CLKFX_MULTIPLY 			of DCM_2: label is 1; 			--  Can be any integer from 1 to 32
attribute CLKFX_MULTIPLY 			of DCM_2: label is 5; 		--  Can be any integer from 2 to 32
attribute CLKFX_DIVIDE 				of DCM_2: label is 2;   		--  Can be any integer from 1 to 32
attribute CLKIN_PERIOD 				of DCM_2: label is "25.0";		--  Specify period of input clock
------------------
-- Signal declare
------------------
signal Gnd: STD_LOGIC;
signal clock2xsig, clock40_sig: STD_LOGIC;
signal clock_in_Bufg: STD_LOGIC;
signal DCM_1_CLK0, DCM_1_CLK0_Bufg: STD_LOGIC;
signal DCM_1_CLK90, DCM_1_CLK180, DCM_1_CLK270, DCM_1_CLK2X, DCM_1_CLKFX, DCM_1_CLKDV: STD_LOGIC;
signal DCM_1_CLK90_Bufg, DCM_1_CLK180_Bufg, DCM_1_CLK270_Bufg, DCM_1_Locked_sig: STD_LOGIC;
signal DCM_2_CLK0, DCM_2_CLK0_Bufg,DCM_2_Locked: STD_LOGIC;
signal DCM_2_CLK90, DCM_2_CLK180, DCM_2_CLK270, DCM_2_CLK2X, DCM_2_CLKFX, DCM_2_CLKDV: STD_LOGIC;
signal DCM_2_CLK90_Bufg, DCM_2_CLK180_Bufg, DCM_2_CLK270_Bufg, DCM_2_Locked_sig: STD_LOGIC;
-------------------------------
----------------------------------------
begin

Gnd <= '0';


clock80			<=	 DCM_1_CLK0_Bufg;
clock40			<=  clock40_sig;	
DCM_1_Locked	<=  DCM_1_Locked_sig and DCM_2_Locked_sig;									
--------------------------------------------------------------------------
--   clock40 DCM                                                      --
--------------------------------------------------------------------------

DCM_1: DCM port map( 	CLKIN 	=> clock_in_Bufg,						
								CLKFB 	=> DCM_1_CLK0_Bufg,
								DSSEN 	=> Gnd,
								PSINCDEC => Gnd,
								PSEN 		=> Gnd,
								PSCLK 	=> Gnd,
								RST 		=> Gnd,	-- Jul-14-06 Was connect to RESET (DIO96_Reset) => clock is stop
								CLK0 		=> DCM_1_CLK0,
								CLK90		=> DCM_1_CLK90,
								CLK180 	=> DCM_1_CLK180,
								CLK270 	=> DCM_1_CLK270,
								CLK2X  	=> DCM_1_CLK2X,
								CLKDV	   => DCM_1_CLKDV,
								CLKFX		=> DCM_1_CLKFX,
								LOCKED 	=> DCM_1_Locked_sig
								);

IBUFG_clock_in: BUFG port map(	I => clock_in,
											O => clock_in_Bufg
										);

BUFG_DCM_1_CLKFB: BUFG port map(	I => DCM_1_CLK0,
											O => DCM_1_CLK0_Bufg
										);
										
--BUFG_DCM_1_CLKFX: BUFG port map(	I => DCM_1_CLKFX,
--BUFG_DCM_1_clock2x: BUFG port map(	I => DCM_1_CLK2X,
--BUFG_DCM_1_clock2x: BUFG port map(	I => DCM_1_CLK0,
--										O => clock80
--										);

BUFG_DCM_1_CLKDV: BUFG port map(	I => DCM_1_CLKDV,
											O => clock40_sig
										);

--------------------------------------------------------------------------
--   clock40_TDC DCM                                                      --
--------------------------------------------------------------------------
DCM_2: DCM port map( 	CLKIN 	=> clock40_sig,	 					
								CLKFB 	=> DCM_2_CLK0_Bufg,
								DSSEN 	=> Gnd,
								PSINCDEC => Gnd,
								PSEN 		=> Gnd,
								PSCLK 	=> Gnd,
								RST 		=> Gnd,	-- Jul-14-06 Was connect to RESET (DIO96_Reset) => clock is stop
								CLK0 		=> DCM_2_CLK0,
								CLK90		=> DCM_2_CLK90,
								CLK180 	=> DCM_2_CLK180,
								CLK270 	=> DCM_2_CLK270,
								CLK2X  	=> DCM_2_CLK2X,
								CLKDV	   => DCM_2_CLKDV,
								CLKFX		=> DCM_2_CLKFX,
								LOCKED 	=> DCM_2_Locked_sig
								);
 
BUFG_DCM_2_CLKFB: BUFG port map(	I => DCM_2_CLK0,
											O => DCM_2_CLK0_Bufg
										);

-- To change the Phase between the clock40 & clock40_TDC, one only need to change the input of this BUFG										
BUFG_DCM_2_clock40_TDC: BUFG port map(	I =>DCM_2_CLK270,
											O => clock40_TDC
										);
 


										 
end Behavioral;
