-- Title: PET_TDC_Controller
-- Description: 
--
-- synthesize:	This design is synthesized at 144 MHz using the Spartant 3 XC3S1000-4fg456,
-- Modules used:
--		1) TOP_TDC_Controller_PAD.vhd (for IO standard interface only)
--		2) TOP_TDC_Controller.vhd (this top level one)
--		3) Clock_generator.vhd 	 (this generate 80 MHz/100 NHz clock from the 40 MHz reference input clock)
--	 	4) fifoctlr_ic_v2.vhd
--	 
-- Authors: Chinh Vu/ Dionisio Doering (Electronic Engineering Department)
-- Date	 : Dec-5-2005
-- Ver	 : 0
-- Rev	 : 0
-----------------
-- Date	 : Dec-16-2005
-- Ver	 : 0
-- Rev	 : 1
-- 	1) Add the FIFO and control to allow the TDC parallel data readout using DIO96 board 
-- 	2) 
-- Date	 : Jan-24-2006
-- 	1) Add 10 more pulses to group 1 for a total of 11 to test readout of the TDC
--			Group 2 & 3 still have only one pulse each 
-- Date	 : Mar-21-2006
-- 	1) Change the control of TDC_Reset (TDC reset signal) to the same as Bunch_Reset to reset
--			All the FIFO Buffer at the beginning of data acquisition  
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--   ____  ____ 
--  /   /\/   / 
-- /___/  \  /    Vendor: Xilinx XC3S1000-4fg456 
-- \   \   \/     Version : 6.3.02i
--  \   \         Application : HPTDC Control and readout in HPTDC Test Board   
--  /   /         Filename : 
-- /___/   /\     Timestamp : 
-- \   \  /  \ 
--  \___\/\___\ 
--
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
library UNISIM;
use UNISIM.VComponents.all;

entity TOP_TDC_Controller is
    Port (
-- Clock & Reset 
			  FPGA_CLK   		: in 	std_logic;
--			  FPGA_RESET     	: in 	std_logic;
			  FE_Test_Pulse	: out std_logic;
-- HPTDC ports: (All Signals name are referenced to the HPTDC)		 
	 		  Par_data_out 	: in 	std_logic_vector(31 downto 0);
           aux_clock 		: out std_logic;
           bunch_reset 		: out std_logic;
           data_ready 		: in 	std_logic;
           encoded_control : out std_logic;
           TDC_error 		: in 	std_logic;
           event_reset 		: out std_logic;
           get_data 			: out std_logic;
           TDC_reset 		: out std_logic;
           serial_bypass_in 	: out std_logic;
           serial_in			: out std_logic;
           serial_out 		: in 	std_logic;
           serial_out_TTL 	: in 	std_logic;
           strobe_out 		: in 	std_logic;
           strobe_out_TTL 	: in 	std_logic;
           test 				: in 	std_logic;
           token_bypass_in : out std_logic;
           token_in  		: out std_logic;
           token_out			: in 	std_logic;
           token_out_TTL 	: in 	std_logic;
           trigger 			: out std_logic;
			  HPTDC_CLK2 		: out std_logic;
-- Logic Analyzer:			  
			  output_clock 	: out std_logic;
			  FPGA_data_out 	: out std_logic_vector(31 downto 0);
			  control_signal_clock 	: out std_logic;
-- PC control ports:		
			  PC_control	   : in  std_logic_vector(8 downto 1);	
			  Spare	  			: out std_logic_vector(8 downto 1) 
			  );
end TOP_TDC_Controller;

architecture Behavioral of TOP_TDC_Controller is

-------------------------------------------------------------------
component Clock_generator is
    Port ( clock_in 		: in std_logic;	-- input clock = 40 MHz
           clock40		: out std_logic;
			  clock40_tdc  : out std_logic;			  
			  clock80		: out std_logic;			  		  		  
			  DCM_1_Locked	: out std_logic; 	-- high = locked
           RESET 			: in std_logic
			  );
end component;
------------------
component fifoctlr_ic_v2 is
   port (read_clock_in:   IN  std_logic;
         write_clock_in:  IN  std_logic;
         read_enable_in:  IN  std_logic;
         write_enable_in: IN  std_logic;
         fifo_gsr_in:     IN  std_logic;
         write_data_in:   IN  std_logic_vector(35 downto 0);
         read_data_out:   OUT std_logic_vector(35 downto 0);
         full_out:        OUT std_logic;
         empty_out:       OUT std_logic;
         fifostatus_out:  OUT std_logic_vector(3 downto 0));
end component;
--------------------------
component BUFG is
	port (
			I : in std_logic;
			O : out std_logic
			);
end component;
--=================================================================
--SIGNALS
--=================================================================
-- Clock Signals
signal   FPGA_CLK_sig			: std_logic;
signal   bunch_reset_sig		: std_logic;
signal   FE_Test_Pulse_sig		: std_logic;
signal   clock40					: std_logic;
signal   clock80					: std_logic;
signal   DCM_1_Locked_sig		: std_logic;
signal   RESET						: std_logic;
signal	HPTDC_CLK2_sig			: std_logic;
signal   count						: std_logic_vector(11 downto 0);
-- DIO96 Signals:
signal	DIO96_Reset			   : std_logic;		-- map to PC_Control_in
signal	DIO96_Reset_Delay	   : std_logic_vector(1 downto 0);
signal	DIO96_Reset_pulse		: std_logic;
signal	DIO96_StartDaq			: std_logic;		-- map to PC_Control_in
signal	DIO96_StartDaq_Delay	: std_logic_vector(1 downto 0);
signal	DAQ_Ena					: std_logic;
signal	DIO96_FIFO_Ren			: std_logic;		-- map to PC_Control_in
signal	DIO96_FIFO_Ren_Delay	: std_logic_vector(1 downto 0); 
signal	DIO96_Sel_Test			: std_logic;  		-- '1' to select the test data otherwise select real TDC data
signal   DIO96_Sel_Ramp_Pattern		: std_logic;
signal	FIFO_Ren					: std_logic;
signal	FIFO_Wen					: std_logic;
signal	FIFO_Full				: std_logic;		-- map to Spare (out)
signal	FIFO_Full_Latch		: std_logic;		
signal	FIFO_Empty				: std_logic;		-- map to Spare (out) 
signal	FIFO_Empty_Latch		: std_logic;		 
signal	FIFO_0_Count			: std_logic_vector(3 downto 0);
signal	TDC_Par_data			: std_logic_vector(35 downto 0);
signal	Test_data				: std_logic_vector(35 downto 0);
signal	Ramp_Pattern_data		: std_logic_vector(35 downto 0);
signal	FIFO_data_in_Latch	: std_logic_vector(35 downto 0);
signal	FIFO_data_in	 		: std_logic_vector(35 downto 0);
signal	FIFO_data_out			: std_logic_vector(35 downto 0);	
signal	FPGA_CLK_B				: std_logic;
signal	clock						: std_logic;
signal	clock_B					: std_logic;	 
signal	TDC_Par_data_latch_1	: std_logic_vector(35 downto 0);
signal	TDC_Par_data_latch_2	: std_logic_vector(35 downto 0);
signal	Start	: std_logic_vector(35 downto 0);
signal	Stop	: std_logic_vector(35 downto 0);	 
signal	different	: std_logic_vector(18 downto 0);
signal	valid_coincident 		: std_logic;	  
signal	counter	: std_logic_vector(1 downto 0);
-- for data filter only
type   	state_type is (idle, valid_check, write_start, write_stop);
signal 	state : state_type;
---**************
begin
--			  RESET		         <= PC_control(1) and  (not DCM_1_Locked_sig) ;	
			  
-- Map to ouput of DIO96 board ports:
			  RESET		         <= DIO96_Reset_pulse;
			  DIO96_Reset			<= PC_control(3);
			  DIO96_StartDaq		<= PC_control(4);		 
			  DIO96_FIFO_Ren		<= PC_control(5);
			  DIO96_Sel_Test		<= PC_control(6);
			  DIO96_Sel_Ramp_Pattern  <= PC_control(8);
--			  DIO96_Sel_Test		<= '1';

-- Map to ouput of HPTDC parallel port:
--   		  TDC_Par_data		 	<= "0000"&Par_data_out;

 
			  aux_clock 			<= '0';
--           bunch_reset_sig 			<= '0';          
           encoded_control 	<= '1';           
           event_reset 			<= '0';
--           get_data 				<= '0';
--           TDC_reset 			<= '0';
           serial_bypass_in 	<= '0';
           serial_in				<= '0';
           token_bypass_in 	<= '0';
                     
           trigger 				<= '0';
--			  HPTDC_CLK2			<= clock40;
			  
-- Logic Analyzer:			  
			  output_clock 		<= clock40;
			  control_signal_clock 	<= '0';
-- PC control ports:		
			  Spare(1)				<= DCM_1_Locked_sig;			  
			  Spare(2)  		   <=  (data_ready or TDC_error or serial_out or serial_out_TTL or strobe_out or strobe_out_TTL or test or token_out or token_out_TTL)		;
			  --token_in 				<=   '1';	  --added
			  Spare(3)				<= data_ready;
--			  Spare(4) is used to monitor the count				
--			  Spare(5)           <= clock80;
			  Spare(5)				<= (PC_control(1) and PC_control(2) and PC_control(6) and PC_control(7) and PC_control(8));
			  Spare(6)           <= '0';
			  Spare(7)				<= FIFO_Full;			-- To DIO96
			  Spare(8)				<= FIFO_Empty ;		-- To DIO96
--- Temporary set FIFO_WEN:
--			  FIFO_Wen				<=	data_ready and DAQ_Ena;
			  FPGA_CLK_B			<=	not FPGA_CLK;
			  clock_B				<=	not clock;
			  clock			      <= clock40;
--			  HPTDC_CLK2_sig		<= clock40;
--			  HPTDC_CLK2			<= HPTDC_CLK2_sig;
			  bunch_reset		   <= bunch_reset_sig;
			  FE_Test_Pulse		<= bunch_reset_sig;

			  different <= "00"&(TDC_Par_data_latch_1(18 downto 2) - TDC_Par_data_latch_2(18 downto 2));
-----------------------------
-- PROCESSES:
-----------------------------
----------------------------------------------------------------------
-- DIO96_input_mapping_process
----------------------------------------------------------------------
DIO96_input_mapping_process: process (RESET, clock80, FPGA_CLK, PC_control)
begin									
	-- Map to input of DIO96 board ports:
	case PC_control(2 downto 1) is 
      when "00" =>
         FPGA_data_out 		<= FIFO_data_out(31 downto 0);
      when "01" =>
         FPGA_data_out 		<= FIFO_data_out(31 downto 28)&FIFO_Wen&FIFO_Ren&FPGA_CLK&FPGA_CLK_B&FIFO_data_out(23 downto 0);
      when "10" =>
         FPGA_data_out 		<= TDC_Par_data(31 downto 28)&FIFO_Wen&FIFO_Ren&FPGA_CLK&FPGA_CLK_B& TDC_Par_data(23 downto 0);
      when "11" =>
         FPGA_data_out 		<= TDC_Par_data(31 downto 0);
      when others =>        
   end case;	
end process;
----------------------------------------------------------------------
-- Delay input signal & Create a Sync pulse from it
----------------------------------------------------------------------
DIO96_Reset_process: process (RESET, clock80, clock)
begin									
	if  clock'event and clock='1' then
		DIO96_Reset_Delay(0) 	<= DIO96_Reset;
		DIO96_Reset_Delay(1)		<= DIO96_Reset_Delay(0);
		if (DIO96_Reset_Delay = "01" and DIO96_Reset = '1') then
			DIO96_Reset_pulse 	<= '1';	
		else
			DIO96_Reset_pulse		<= '0';	
		end if;		
	end if;
end process;
----------------------------------------------------------------------
DIO96_StartDaq_process: process (RESET, clock80, clock, FIFO_Full, FIFO_0_Count)
begin
	if (RESET = '1') then		
		DAQ_Ena 		<= '0';												
	elsif  clock'event and clock='1' then
		DIO96_StartDaq_Delay(0) 	<= DIO96_StartDaq;
		DIO96_StartDaq_Delay(1)		<= DIO96_StartDaq_Delay(0);
		FIFO_Full_Latch  				<= FIFO_Full;
		if (DIO96_StartDaq_Delay = "01" and DIO96_StartDaq = '1') then
			DAQ_Ena 		<= '1';
		-- if PC_control(7)= '1' then the DAQ run all the time (ignoring the FIFO_Full signal) 	
		elsif (FIFO_Full = '1' and PC_control(7)= '0') then
--		elsif (FIFO_0_Count = "0110") then
			DAQ_Ena 		<= '0';	
		end if;
	end if;
end process;
-------------------------------------------------------------
FIFO_Wen_process: process (RESET, clock80, clock, data_ready)
begin
	if (RESET = '1') then		
		get_data 		<= '0';
		FIFO_data_in_Latch  <= 	"000000000000000000000000000000000000";
		TDC_Par_data_latch_1	<= "000000000000000000000000000000000000";
		TDC_Par_data_latch_2	<= "000000000000000000000000000000000000";
		start		<= "000000000000000000000000000000000000";
		stop		<= "000000000000000000000000000000000000";
		valid_coincident <= '0';	
		counter 			<= "00";
		FIFO_Wen    	<= '0';
		state				<= idle;											
	elsif  clock'event and clock='0' then		-- Used to be Falling edge July-17-06
		if PC_control(1) = '1' then
		-- Add the new data filter for pattern (chan4 then chan5 with chan5-chan4 < 2048)
		-- Latch the TDC data
			case state is
				when idle =>
					FIFO_Wen    	<= '0';

					if (data_ready = '1' and DAQ_Ena ='1') then
						get_data 		<= '1';
						TDC_Par_data_latch_1	<= "0000"&Par_data_out;
						TDC_Par_data_latch_2	<= TDC_Par_data_latch_1;
						state <=  valid_check;
					else
						get_data 		<= '0';
					end if;

				when valid_check =>
					get_data 		<= '0'; -- to prevent further read in data from TDC 
				-- Check the sequence channel 4 then 5 and Calculate the difference in data
				--	different <= "00"&(TDC_Par_data_latch_1(18 downto 2) - TDC_Par_data_latch_2(18 downto 2));
					if ((TDC_Par_data_latch_2(23 downto 21)= "100") and (TDC_Par_data_latch_1(23 downto 21)= "101")) and different( 18 downto 9) = "000000000" then
						start <= TDC_Par_data_latch_2;
						stop	<= TDC_Par_data_latch_1;
						state <= write_start;
					else
						state <= idle;
						FIFO_Wen    	<= '0';
					end if;
																
				when write_start =>
					TDC_Par_data	<= start;							
					FIFO_Wen    	<= '1';
					state <= write_stop;
				when write_stop =>
					TDC_Par_data	<= stop;							
					FIFO_Wen    	<= '1';
					state <= idle;
				when others =>
					FIFO_Wen    	<= '0';
					TDC_Par_data   <= "000000000000000000000000000000000000";	
					state <= idle;
			end case;

		else			
-- write to FIFO the regular way upto Sept-26-2006
			if (data_ready = '1' and DAQ_Ena ='1') then
				get_data 		<= '1';
				FIFO_Wen    	<= '1';
				TDC_Par_data	<= "0000"&Par_data_out;
			else
				get_data 		<= '0';
				FIFO_Wen    	<= '0';
				TDC_Par_data   <= "000000000000000000000000000000000000";	
			end if;
		end if;
	end if;
end process;
--------------------------------------------------------------------------------------
-- Make sure the DIO96_FIFO_Ren pulse is at least 2 of the system clock cycles 
--------------------------------------------------------------------------------------
DIO96_FIFO_Ren_process: process (RESET, clock80, clock, DIO96_FIFO_Ren, FIFO_Full)
begin
	if (RESET = '1') then		
		FIFO_Ren 		<= '0';												
	elsif  clock'event and clock='1' then
		DIO96_FIFO_Ren_Delay(0) 	<= DIO96_FIFO_Ren;
		DIO96_FIFO_Ren_Delay(1)		<= DIO96_FIFO_Ren_Delay(0);
		if (DIO96_FIFO_Ren_Delay = "01" and DIO96_FIFO_Ren = '1') then
			FIFO_Ren 		<= '1';	
		else
			FIFO_Ren 		<= '0';	
		end if;
	end if;
end process;
-------------------------------------------------------------
Ramp_pattern_process: process (RESET, clock80, clock, data_ready)
begin
	if ((RESET = '1') or (DAQ_Ena = '0')) then		
		Ramp_pattern_data 		<= "000000000000000000000000000000000000";												
	elsif  clock'event and clock='0' then
		if (FIFO_Wen = '1') then
			Ramp_pattern_data 	<=  Ramp_pattern_data + '1';
		else
			Ramp_pattern_data 	<=  Ramp_pattern_data ;	
		end if;
	end if;
end process;
----------------------------------------------------------------------
Synchronous_pulsing_process: process (RESET, clock80, clock)
begin
	if (RESET = '1' or DAQ_Ena = '0') then
--   if (RESET = '1') then
		count				<= "000000000000";
		bunch_reset_sig 	<= '1';
		TDC_reset		<= '1';  -- C.Vu Added mar-21-06 to reset all the FIFO Buffer
--		get_data 		<= '0';
		FE_Test_Pulse_sig  <= '0';
		token_in 		<=   '1';	  --added									
		
--	elsif  clock80'event and clock80='1' then
	elsif  clock'event and clock='1' then
			if count = "000000101001" then 
				count	<= "000000000000";
			else
				count	<= count + '1';
			end if;
			
			if Par_data_out = "00000000000000000000000000000000" then
				Spare(4)  <= '0';
			else
				Spare(4)  <= '1'; 
			end if;
		case count is
			when "000000000000" =>
				bunch_reset_sig 	<= '1';  -- reset ='1'
--				TDC_reset		<= '1';  -- C.Vu Added mar-21-06 to reset all the FIFO Buffer
				FE_Test_Pulse_sig  <= '0';
				token_in 		<= '1';
--			when "000000000001" =>		-- 2 Seng Changed aug-07-06
--				TDC_reset		<= '0';  -- C.Vu Added mar-21-06; Seng Changed aug-08-06	
			when "000000000001" =>		-- 2 Seng Changed aug-07-06 (bunch reset must be within 1 clock cycle, 25 ns)
				bunch_reset_sig 	<= '0';
				TDC_reset		<= '0';  -- C.Vu Added mar-21-06; Seng Changed aug-08-06
				FE_Test_Pulse_sig  <= '0';
-- Pulse #1
			when "000000010010" =>		-- 18 delay 10 counts of 25 ns =250 ns
				FE_Test_Pulse_sig  <= '1';
			when "000000010011" =>		--19 
				FE_Test_Pulse_sig  <= '0';
--			when "000000010110" =>		-- 22 
--				FE_Test_Pulse_sig  <= '1';
--			when "000000010111" =>		--23
--				FE_Test_Pulse_sig  <= '0';	
--			when "000000011010" =>		-- 26
--				FE_Test_Pulse_sig  <= '1';		
--			when "000000011011" =>		--27
--				FE_Test_Pulse_sig  <= '0';
--			when "000000011110" =>		-- 30 
--				FE_Test_Pulse_sig  <= '1';
--			when "000000011111" =>		--31
--				FE_Test_Pulse_sig  <= '0';	
--			when "000000100010" =>		-- 34 
--				FE_Test_Pulse_sig  <= '1';
--			when "000000100011" =>		--35
--				FE_Test_Pulse_sig  <= '0';	
--			when "000000100110" =>		-- 38
--				FE_Test_Pulse_sig  <= '1';
--			when "000000100111" =>		--39 
--				FE_Test_Pulse_sig  <= '0';
--			when "000000101010" =>		-- 42
--				FE_Test_Pulse_sig  <= '1';	
--			when "000000101011" =>		--43 
--				FE_Test_Pulse_sig  <= '0';
--			when "000000101110" =>		-- 46
--				FE_Test_Pulse_sig  <= '1';
--			when "000000101111" =>		--47 
--				FE_Test_Pulse_sig  <= '0';
--			when "000000110010" =>		-- 50
--				FE_Test_Pulse_sig  <= '1';	
--			when "000000110011" =>		--51 
--				FE_Test_Pulse_sig  <= '0';
--			when "000000110110" =>		-- 54
--				FE_Test_Pulse_sig  <= '1';	
--			when "000000110111" =>		--55 
--				FE_Test_Pulse_sig  <= '0';
--			when "000000111010" =>		-- 58
--				FE_Test_Pulse_sig  <= '1';
--			when "000000111011" =>		--59 
--				FE_Test_Pulse_sig  <= '0';
							
-- Pulse #2																		
			when "010000010010" =>		-- 1042 Second pulse set
				bunch_reset_sig 	<= '0';
				FE_Test_Pulse_sig  <= '1';
			when "010000010101" =>		-- 1045 
				bunch_reset_sig 	<= '0';
				FE_Test_Pulse_sig  <= '0';
			when "010000010111" =>		-- 1047 
				bunch_reset_sig 	<= '0';
				FE_Test_Pulse_sig  <= '0';
			when "010000011110" =>		-- 1054
				bunch_reset_sig 	<= '0';
				FE_Test_Pulse_sig  <= '0';
--				get_data 		<= '1';	--added
			when "010000100010" =>		-- 1058
				bunch_reset_sig 	<= '0';
				FE_Test_Pulse_sig  <= '0';
--				get_data 		<= '0';	--added	
-- Pulse #3																				
			when "011100010010" =>		-- 2834 Third pulse set
				bunch_reset_sig 	<= '0';
				FE_Test_Pulse_sig  <= '1';
			when "011100010101" =>		-- 2835 
				bunch_reset_sig 	<= '0';
				FE_Test_Pulse_sig  <= '0';
			when "011100011110" =>		-- 2846
				bunch_reset_sig 	<= '0';
				FE_Test_Pulse_sig  <= '0';
--				get_data 		<= '1';	--added
			when "011100100010" =>		-- 2850
				bunch_reset_sig 	<= '0';
				FE_Test_Pulse_sig  <= '0';
--				get_data 		<= '0';	--added														
			when others => 
				
		end case;
	end if;
end process;

----------------------------------------------------------------------
Test_data_process: process (RESET, clock80, clock, DIO96_Sel_Test, TDC_Par_data)
begin
	if (RESET = '1' or DAQ_Ena = '0') then
		Test_data 	<= "000000000000000000000000000000000000";
	elsif  clock'event and clock='1' then
			if DIO96_Sel_Test = '0' then
				FIFO_data_in  <= TDC_Par_data;
			else
				if DIO96_Sel_Ramp_Pattern = '0' then
					FIFO_data_in  <= Test_data;
				else
					FIFO_data_in  <= Ramp_Pattern_data; 
				end if;
			end if;
		case FIFO_0_Count is
		-- Test_data format is spare(4 bits)&leading edje(4 bits)&chip_ID(4 bits)&chan(5 bits)&data(19 bits)
			when "0000" =>			
				Test_data 	<= "0000"&"0100"&"0001"&"00001"&"0000000000000000000";
			when "0001" =>
				Test_data 	<= "0000"&"0100"&"0001"&"00001"&"0000000000000000001";
			when "0010" =>
				Test_data 	<= "0000"&"0100"&"0001"&"00001"&"0000000000000000010";
			when "0011" =>
				Test_data 	<= "0000"&"0100"&"0001"&"00001"&"0000000000000000011";
			when "0100" =>
				Test_data 	<= "0000"&"0100"&"0001"&"00001"&"0000000000000000100";
			when "0101" =>
				Test_data 	<= "0000"&"0100"&"0001"&"00001"&"0000000000000000101";	
			when "0110" =>
				Test_data 	<= "0000"&"0100"&"0001"&"00001"&"0000000000000000110";	
			when "0111" =>
				Test_data 	<= "0000"&"0100"&"0001"&"00001"&"0000000000000000111";
			when "1000" =>
				Test_data 	<= "0000"&"0100"&"0001"&"00001"&"0000000000000001000";					
			when "1001" =>
				Test_data 	<= "0000"&"0100"&"0001"&"00001"&"0000000000000001001";
			when "1010" =>
				Test_data 	<= "0000"&"0100"&"0001"&"00001"&"0000000000000001010";
			when "1011" =>
				Test_data 	<= "0000"&"0100"&"0001"&"00001"&"0000000000000001011";
			when "1100" =>
				Test_data 	<= "0000"&"0100"&"0001"&"00001"&"0000000000000001100";
			when "1101" =>
				Test_data 	<= "0000"&"0100"&"0001"&"00001"&"0000000000000001101";
			when "1110" =>
				Test_data 	<= "0000"&"0100"&"0001"&"00001"&"0000000000000001110";					
			when "1111" =>
				Test_data 	<= "0000"&"0100"&"0001"&"00001"&"0000000000000001111";											
			when others => 
				
		end case;
	end if;
end process;
----------------------------------------------------------------------
HPTDC_CLK_process: process (RESET, clock80, FPGA_CLK)
begin
	if (RESET = '1') then		
--		HPTDC_CLK2_sig 		<= '0';												
	elsif  FPGA_CLK'event and FPGA_CLK='1' then
		HPTDC_CLK2_sig 		<= not HPTDC_CLK2_sig;
	end if;
end process;
--------------------------------------------	
-- Component instantiation:
--------------------------------------------						  
-- Clock generator:
Clock_generator_1: Clock_generator port map(
			clock_in 			=>  FPGA_CLK,				-- input clock  = 80 MHz
         clock40				=>  clock40,				-- output clock = 40 MHz
			clock40_tdc       =>  HPTDC_CLK2,		  
			clock80				=>  clock80,				-- output clock = 80 MHz	  		  		  
			DCM_1_Locked		=>  DCM_1_Locked_sig,	-- high = locked
         RESET 				=>  RESET
		   );
------------------
FIFO_0: fifoctlr_ic_v2            port map(	
			read_clock_in   	=>	clock,
	      write_clock_in  	=>	clock_B, -- OLD is Clock_B July-17-06
		   read_enable_in  	=>	FIFO_Ren,
		   write_enable_in 	=> FIFO_Wen,	
		   fifo_gsr_in     	=> RESET,
		   write_data_in  	=>	FIFO_data_in,	  
		   read_data_out  	=>	FIFO_data_out,	  
		   full_out        	=> FIFO_Full,
		   empty_out       	=>	FIFO_Empty,
		   fifostatus_out  	=>	FIFO_0_Count(3 downto 0)
			);
--BUFG_clock: BUFG port map(	I => HPTDC_CLK2_sig,
--											O => clock
--										);
													 
end Behavioral;
