-- 										***  P.E.T (positron emission tomography) ***
--
-- The SPARTANT 3 LVDS ouput can drive the 100 ohms at external lvds receiver WITHOUT the output resistor network (3 resistor each) 

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
--******************************************************************
--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components i.e IBUFDS, OBUFDS etc....
library UNISIM;
use UNISIM.VComponents.all;
--******************************************************************
entity TOP_TDC_Controller_PAD is
    Port (
	  		  FPGA_CLK		 	: in 	std_logic;
           FPGA_CLK_B 		: in 	std_logic;
			  FE_Test_Pulse	: out std_logic;
-- HPTDC ports: (All Signals name are referenced to the HPTDC)	 
	 		  Par_data_out 	: in 	std_logic_vector(31 downto 0);
           aux_clock 		: out std_logic;
			  aux_clockB 		: out std_logic;
           bunch_reset 		: out std_logic;
			  bunch_resetB 	: out std_logic;
           data_ready 		: in 	std_logic;
           encoded_control : out std_logic;
			  encoded_controlB: out std_logic;
           error 				: in 	std_logic;
           event_reset 		: out std_logic;
			  event_resetB 	: out std_logic;
           get_data 			: out std_logic;
           reset 				: out std_logic;
			  resetB 			: out std_logic;
           serial_bypass_in 	: out std_logic;
			  serial_bypass_inB 	: out std_logic;
           serial_in			: out std_logic;
			  serial_inB		: out std_logic;
           serial_out 		: in 	std_logic;
			  serial_outB 		: in 	std_logic;
           serial_out_TTL 	: in 	std_logic;
           strobe_out 		: in 	std_logic;
			  strobe_outB 		: in 	std_logic;
           strobe_out_TTL 	: in 	std_logic;
           test 				: in 	std_logic;
           token_bypass_in 	: out std_logic;
			  token_bypass_inB	: out std_logic;
           token_in	 		: out std_logic;
			  token_inB 		: out std_logic;
           token_out 		: in 	std_logic;
			  token_outB 		: in 	std_logic;
           token_out_TTL 	: in 	std_logic;
           trigger 			: out std_logic;
			  triggerB 			: out std_logic;			  
			  HPTDC_CLK2		: out std_logic;
			  HPTDC_CLK2_B		: out std_logic;
-- Logic Analyzer:			  
			  output_clock 	: out std_logic;
			  FPGA_data_out 	: out std_logic_vector(31 downto 0);
			  control_signal_clock 	: out std_logic;
-- PC control ports:		
			  PC_control	   : in  std_logic_vector(8 downto 1);	
			  Spare	  			: out std_logic_vector(8 downto 1) 
			  );
end TOP_TDC_Controller_PAD;

architecture Behavioral of TOP_TDC_Controller_PAD is

--component
component OBUFDS is
	port (
			I 	: in 	std_logic;
			O 	: out std_logic;
			OB : out std_logic
			);
end component;
--------------
component IBUFDS is
	port (
			I 	: in 	std_logic;
			IB : in  std_logic;
			O  : out std_logic
			);
end component;
--------------
component TOP_TDC_Controller is
	Port ( 
-- Clock & Reset 
			  FPGA_CLK   		: in 	std_logic;
--			  FPGA_RESET     	: in 	std_logic;
			  FE_Test_Pulse	: out std_logic;
-- HPTDC ports: (All Signals name are referenced to the HPTDC)		 
	 		  Par_data_out 	: in 	std_logic_vector(31 downto 0);
           aux_clock 		: out std_logic;
           bunch_reset 		: out std_logic;
           data_ready 		: in 	std_logic;
           encoded_control : out std_logic;
           TDC_error 		: in 	std_logic;
           event_reset 		: out std_logic;
           get_data 			: out std_logic;
           TDC_reset 		: out std_logic;
           serial_bypass_in 	: out std_logic;
           serial_in			: out std_logic;
           serial_out 		: in 	std_logic;
           serial_out_TTL 	: in 	std_logic;
           strobe_out 		: in 	std_logic;
           strobe_out_TTL 	: in 	std_logic;
           test 				: in 	std_logic;
           token_bypass_in : out std_logic;
           token_in  		: out std_logic;
           token_out			: in 	std_logic;
           token_out_TTL 	: in 	std_logic;
           trigger 			: out std_logic;
			  HPTDC_CLK2 		: out std_logic;

-- Logic Analyzer:			  
			  output_clock 	: out std_logic;
			  FPGA_data_out 	: out std_logic_vector(31 downto 0);
			  control_signal_clock 	: out std_logic;
-- PC control ports:		
			  PC_control	   : in  std_logic_vector(8 downto 1);	
			  Spare	  			: out std_logic_vector(8 downto 1) 
			  );
end component;

--=================================================================
--SIGNALS
--=================================================================
-- Clock Signals
signal   FPGA_CLK_sig			: std_logic;
signal   clock40_sig				: std_logic;
signal   clock32_sig				: std_logic;
signal   FPGA_RESET_sig			: std_logic;
-- HPTDC Signals				 
signal	Par_data_out_sig 		: std_logic_vector(31 downto 0);
signal   aux_clock_sig 			: std_logic;
signal   bunch_reset_sig 		: std_logic;
signal   data_ready_sig			: std_logic;
signal   encoded_control_sig 	: std_logic;
signal   error_sig 				: std_logic;
signal   event_reset_sig 		: std_logic;
signal   get_data_sig 			: std_logic;
signal   TDC_reset_sig 			: std_logic;
signal   serial_bypass_in_sig	: std_logic;
signal   serial_out_sig			: std_logic;
signal   serial_in_sig	 		: std_logic;
signal   serial_out_TTL_sig 	: std_logic;
signal   strobe_out_sig	 		: std_logic;
signal   strobe_out_TTL_sig 	: std_logic;
signal   test_sig 				: std_logic;
signal   token_bypass_in_sig 		: std_logic;
signal   token_out_sig	 		: std_logic;
signal   token_in_sig 			: std_logic;
signal   token_out_TTL_sig 		: std_logic;
signal   trigger_sig	 			: std_logic;
signal	HPTDC_CLK2_sig			: std_logic;
-- Logic Analyzer:			  
signal	output_clock_sig 		: std_logic;
signal	FPGA_data_out_sig 	: std_logic_vector(31 downto 0);
signal	control_signal_clock_sig 	: std_logic;
-- PC control ports:		
signal	PC_control_sig	   	: std_logic_vector(8 downto 1);	
signal	Spare_sig	  			: std_logic_vector(8 downto 1);
---===================================================================
begin
-----------------------------
-- Concurrent statements
----------------------------- 
--Inputs:
						
			Par_data_out_sig	<= Par_data_out;
			data_ready_sig		<= data_ready;
			error_sig			<= error;
			serial_out_TTL_sig	<= serial_out_TTL;
			strobe_out_TTL_sig	<= strobe_out_TTL;
			test_sig				<= test;
			token_out_TTL_sig	<= token_out_TTL;
			PC_control_sig		<= PC_control;
--			FPGA_RESET_sig		<= PC_control_sig(1);

--Outputs:			
			get_data				<= get_data_sig;
			output_clock		<= output_clock_sig;
			FPGA_data_out		<= FPGA_data_out_sig;
				
			Spare					<=	Spare_sig;

	   	control_signal_clock		<= control_signal_clock_sig;

			  
-----------------------------
-- Components Instantiation:
----------------------------- 
TOP_TDC_Controller_1 : TOP_TDC_Controller port map (
			FPGA_CLK				=>  FPGA_CLK_sig,
--			FPGA_RESET     	=>  PC_control_sig(1),
			FE_Test_Pulse   	=>  FE_Test_Pulse,
			Par_data_out 		=>  Par_data_out_sig,
         aux_clock 			=>  aux_clock_sig,
         bunch_reset 		=>  bunch_reset_sig,
         data_ready 			=>  data_ready_sig,
         encoded_control 	=>  encoded_control_sig,
         TDC_error 			=>  error_sig,
         event_reset 		=>  event_reset_sig ,
         get_data 			=>  get_data_sig ,
         TDC_reset 			=>  TDC_reset_sig ,
         serial_bypass_in 	=>  serial_bypass_in_sig,
         serial_out			=>  serial_out_sig,
         serial_in 			=>  serial_in_sig,
         serial_out_TTL 	=>  serial_out_TTL_sig,
         strobe_out 			=>  strobe_out_sig,
         strobe_out_TTL 	=>  strobe_out_TTL_sig,
         test 					=>  test_sig,
         token_bypass_in 	=>  token_bypass_in_sig,
         token_out 			=>  token_out_sig,
         token_in 			=>  token_in_sig,
         token_out_TTL 		=>  token_out_TTL_sig,
         trigger 				=>  trigger_sig,			
			HPTDC_CLK2 			=>  HPTDC_CLK2_sig,
	-- Logic Analyzer:			  
			output_clock 		=>  output_clock_sig,
			FPGA_data_out 		=>  FPGA_data_out_sig,
			control_signal_clock 	=>  control_signal_clock_sig,
	-- PC control ports:		
			PC_control	   	=>  PC_control_sig,
			Spare	  				=>  Spare_sig
			);
 
--------------------------------------------
-- LVDS Input buffer (2.5v LVDS to single)

FPGA_CLK_in1: IBUFDS port map(
									I 	=> FPGA_CLK,
									IB => FPGA_CLK_B,
									O  => FPGA_CLK_sig
									);
			 
serial_in1: IBUFDS port map(
									I 	=> serial_out,
									IB => serial_outB,
									O  => serial_out_sig
									);

strobe_out1: IBUFDS port map(
									I 	=> strobe_out,
									IB => strobe_outB,
									O  => strobe_out_sig
									);

token_in1: IBUFDS port map(
									I 	=> token_out,
									IB => token_outB,
									O  => token_out_sig
									);

--------------------------------------------
-- LVDS Output buffer (single to LVDS 2.5v) 			 
aux_clock1: OBUFDS port map(
									I 	=> aux_clock_sig,
									O 	=> aux_clock,
									OB => aux_clockB
									);

bunch_reset1: OBUFDS port map(
									I 	=> bunch_reset_sig,
									O 	=> bunch_reset,
									OB => bunch_resetB
									);

encoded_control1: OBUFDS port map(
									I 	=> encoded_control_sig,
									O 	=> encoded_control,
									OB => encoded_controlB
									);

event_reset1: OBUFDS port map(
									I 	=> event_reset_sig,
									O 	=> event_reset,
									OB => event_resetB
									);

reset1: OBUFDS port map(
									I 	=> TDC_reset_sig,
									O 	=> reset,
									OB => resetB
									);

serial_bypass1: OBUFDS port map(
									I 	=> serial_bypass_in_sig,
									O 	=> serial_bypass_in,
									OB => serial_bypass_inB
									);

serial_out1: OBUFDS port map(
									I 	=> serial_in_sig,
									O 	=> serial_in,
									OB => serial_inB
									);

token_bypass1: OBUFDS port map(
									I 	=> token_bypass_in_sig,
									O 	=> token_bypass_in,
									OB => token_bypass_inB
									);

token_out1: OBUFDS port map(
									I 	=> token_in_sig,
									O 	=> token_in,
									OB => token_inB
									);

trigger1: OBUFDS port map(
									I 	=> trigger_sig,
									O 	=> trigger,
									OB => triggerB
									);

HPTDC_CLK1: OBUFDS port map(
									I 	=> HPTDC_CLK2_sig,
									O 	=> HPTDC_CLK2,
									OB => HPTDC_CLK2_B
									);
end Behavioral;
