library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;

entity TOP_TDC_Controller is
    Port (
-- Clock & Reset 
			  FPGA_CLK   		: in 	std_logic;
--			  FPGA_RESET     	: in 	std_logic;
			  FE_Test_Pulse	: out std_logic;
-- HPTDC ports: (All Signals name are referenced to the HPTDC)		 
	 		  Par_data_out 	: in 	std_logic_vector(31 downto 0);
           aux_clock 		: out std_logic;
           bunch_reset 		: out std_logic;
           data_ready 		: in 	std_logic;
           encoded_control : out std_logic;
           TDC_error 		: in 	std_logic;
           event_reset 		: out std_logic;
           get_data 			: out std_logic;
           TDC_reset 		: out std_logic;
           serial_bypass_in 	: out std_logic;
           serial_in			: out std_logic;
           serial_out 		: in 	std_logic;
           serial_out_TTL 	: in 	std_logic;
           strobe_out 		: in 	std_logic;
           strobe_out_TTL 	: in 	std_logic;
           test 				: in 	std_logic;
           token_bypass_in : out std_logic;
           token_in  		: out std_logic;
           token_out			: in 	std_logic;
           token_out_TTL 	: in 	std_logic;
           trigger 			: out std_logic;
			  HPTDC_CLK2 		: out std_logic;
-- Logic Analyzer:			  
			  output_clock 	: out std_logic;
			  FPGA_data_out 	: out std_logic_vector(31 downto 0);
			  control_signal_clock 	: out std_logic;
-- PC control ports:		
			  PC_control	   : in  std_logic_vector(8 downto 1);	
			  Spare	  			: out std_logic_vector(8 downto 1) 
			  );
end TOP_TDC_Controller;

architecture Behavioral of TOP_TDC_Controller is

-------------------------------------------------------------------
component Clock_generator is
    Port ( clock_in 		: in std_logic;	-- input clock = 40 MHz
           clock40		: out std_logic;			  
			  clock32		: out std_logic;			  		  		  
			  DCM_1_Locked	: out std_logic; 	-- high = locked
           RESET 			: in std_logic
			  );
end component;
--=================================================================
--SIGNALS
--=================================================================
-- Clock Signals
signal   FPGA_CLK_sig			: std_logic;
signal   clock40					: std_logic;
signal   clock32					: std_logic;
signal   DCM_1_Locked_sig		: std_logic;
signal   RESET						: std_logic;
signal   count						: std_logic_vector(11 downto 0);

---**************
begin
--			  RESET		         <= PC_control(1) and  (not DCM_1_Locked_sig) ;	
			  RESET		         <= PC_control(1)  ;
			  FPGA_data_out 		<= Par_data_out;
			  aux_clock 			<= '0';
--           bunch_reset 			<= '0';          
           encoded_control 	<= '1';           
           event_reset 			<= '0';
           get_data 				<= '0';
           TDC_reset 			<= '0';
           serial_bypass_in 	<= '0';
           serial_in				<= '0';
           token_bypass_in 	<= '0';
                     
           trigger 				<= '0';
			  HPTDC_CLK2			<= '0';
-- Logic Analyzer:			  
			  output_clock 		<= '0';
			  control_signal_clock 	<= '0';
-- PC control ports:		
			  
			  Spare(8 downto 2)	<= PC_control(8 downto 2) ;
			  Spare(1)				<= DCM_1_Locked_sig;

			  token_in  		   <=  (data_ready or TDC_error or serial_out or serial_out_TTL or strobe_out or strobe_out_TTL or test or token_out or token_out_TTL)		;
-----------------------------
-- PROCESSES:
-----------------------------
Synchronous_pulsing_process: process (RESET, clock40, FPGA_CLK)
begin
	if (RESET = '1') then
		count				<= "000000000000";
		bunch_reset 	<= '1';
		FE_Test_Pulse  <= '0';
--	elsif  clock40'event and clock40='1' then
	elsif  FPGA_CLK'event and FPGA_CLK='1' then
		count	<= count + '1';
		case count is
			when "000000000000" =>
				bunch_reset 	<= '1';  -- reset ='1'
				FE_Test_Pulse  <= '0';
			when "000000001000" =>		-- 8
				bunch_reset 	<= '0';
				FE_Test_Pulse  <= '0';
			when "000000010010" =>		-- 18 delay 10 counts of 25 ns =250 ns
				bunch_reset 	<= '0';
				FE_Test_Pulse  <= '1';
			when "000000011010" =>		-- 26
				bunch_reset 	<= '0';
				FE_Test_Pulse  <= '0';					
			when others => 
				
		end case;
	end if;
end process;
--------------------------------------------	
-- Component instantiation:
--------------------------------------------						  
-- Clock generator:
Clock_generator_1: Clock_generator port map(
			clock_in 			=>  FPGA_CLK,				-- input clock  = 40 MHz
         clock40				=>  clock40,				-- output clock = 40 MHz	  
			clock32				=>  clock32,				-- output clock = 32 MHz	  		  		  
			DCM_1_Locked		=>  DCM_1_Locked_sig,	-- high = locked
         RESET 				=>  RESET
		   );
end Behavioral;
